VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CResizer"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'This is a rule based resizer that automatically resizes the conrtols on the tartget form you specify
'You do not need to modify this code unless you want to add a few rules
'If you set the TargetForm property after the target form has been resized, you will find that _
the controls do not follow the rules. They retain their original sizes. In this case, you will need to call the _
Resize method of the class/object. This will then enforce the rules and then everything should be OK

'The available rules
Public Enum ResizerRules
    SizeTo_Width
    SizeTo_Right
    SizeTo_Bottom
    Align_RightToRight
End Enum

'each instance of a rule
Private Type RULE_INSTANCE
    Rule As ResizerRules
    Source As Object
    Dest() As Variant
    Border As Long
End Type

'the size of each control/source object for further calculations
Private Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

Private WithEvents frm As Form              'keep a reference to the form that we have to watch
Attribute frm.VB_VarHelpID = -1
Private allInst() As RULE_INSTANCE          'keep all instances of all rules to enforce when frm resizes

'manually enforce the rules to make the controls behave themselves
Public Sub Resize()
    frm_Resize
End Sub

'Set the target form. There is no need to find out which form is target form from the classes clien so there is _
no property get procedure to return a reference to the target form. Always ensure that each form has its own instance _
of the CResizer class. It should then set the target form to itself so that it's child controls are resized when it _
changes size.
Public Property Set TargetForm(Target As Form)
    Set frm = Target
End Property

'allow the client form to add an instance of a rule.
'Rule       the rule in question
'Source     the object the rule is being applied in relation to. For instance, if the rule is align rights, source is _
            the object to whose right the rest of the controlw will be aligned
'Border     how much space should we leave when applying the rules. Size to width with a border will have the border _
            around the destination which will be Source.Width - (Border * 2) in width
'Dest()     one or more target controls. These are the controls that will have to follow the applied rules.
Public Sub AddRule(Rule As ResizerRules, Source As Object, Border As Long, ParamArray Dest() As Variant)
    Dim lng As Long
    On Error Resume Next
    
    lng = UBound(allInst)
    If lng = 0 Then
        ReDim allInst(1 To 1)
    Else
        ReDim Preserve allInst(1 To lng + 1)
    End If
    
    With allInst(lng + 1)
        .Rule = Rule
        Set .Source = Source
        .Dest = Dest
        .Border = Border
    End With
End Sub

'put code here to initialize if you extend this class
Private Sub Class_Initialize()
    'anything that needs to be done at startup
End Sub

'this is the class responding to the target form's Resize event. You do not need to put any code inside the form's _
own resize event to handle these issues.
Private Sub frm_Resize()
    Dim lng As Long
    
    'loop through all Rules and Apply them
    For lng = LBound(allInst) To UBound(allInst)
        ApplyRule allInst(lng)
    Next
End Sub

'This is where each instance of a rule is applied.
Private Sub ApplyRule(Rule As RULE_INSTANCE)
    On Error Resume Next
    Dim srcExtents As RECT
    Dim lng As Long, lngBorder As Long
    
    'determine the bounding rectangle
    With Rule
        If TypeOf .Source Is Form Then
            srcExtents.Left = 0
            srcExtents.Top = 0
            srcExtents.Right = .Source.ScaleWidth
            srcExtents.Bottom = .Source.ScaleHeight
        Else
            srcExtents.Left = .Source.Left
            srcExtents.Top = .Source.Top
            srcExtents.Right = .Source.Left + .Source.Width
            srcExtents.Bottom = .Source.Top + .Source.Height
        End If
        lngBorder = .Border
        
        'apply rule to all controls in the Dest() array
        For lng = 0 To UBound(.Dest)
            With .Dest(lng)
                Select Case Rule.Rule
                Case ResizerRules.SizeTo_Width
                    .Left = lngBorder
                    .Width = srcExtents.Right - srcExtents.Left - lngBorder * 2
                Case ResizerRules.SizeTo_Right
                    .Width = srcExtents.Right - .Left - lngBorder
                Case ResizerRules.SizeTo_Bottom
                    .Height = srcExtents.Bottom - .Top - lngBorder
                Case ResizerRules.Align_RightToRight
                    .Left = srcExtents.Right - .Width - lngBorder
                End Select
            End With
        Next
    End With
End Sub

'clean up your memory here by destroying all object references that may still be active. Yes, VB knows how to deal _
with memory. Don't buy all the story's about garbage collectors in other languages. If you leave this out, VB will _
still clean it up for you. But it is always safer and more ideal to clean it up yourself and that way you do not _
have anything taking any uneccessary memory. Happy coding. Fabulous Fadz.
Private Sub frm_Terminate()
    Set frm = Nothing
End Sub

